/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.user;

import static org.assertj.core.api.Assertions.*;

import java.util.List;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Spy;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.user.settings.UserSettingsResource;

class UserProfileResourceAssemblerTest {

	@Spy
	private UserProfileResourceAssembler assembler;

	private static final String USER_MANAGER_URL = LoremIpsum.getInstance().getUrl();

	@DisplayName("To Userprofile Resource")
	@Nested
	class TestToUserProfileResource {

		@Test
		void shouldAddSelfLink() {
			var res = assembler.toUserProfileResource(UserTestFactory.create(), USER_MANAGER_URL);

			assertThat(res.getLinks().get(UserProfileResourceAssembler.REL_SELF))
					.isNotNull()
					.hasFieldOrPropertyWithValue("href",
							USER_MANAGER_URL + UserProfileResource.USER_PROFILE_RESOURCE_PATH_TEMPLATE.replace("{id}", UserTestFactory.ID_STR));
		}

		@Test
		void shouldAddSettingsLink() {
			var res = assembler.toUserProfileResource(UserTestFactory.create(), USER_MANAGER_URL);

			assertThat(res.getLinks().get(UserProfileResourceAssembler.REL_SETTINGS))
					.isNotNull()
					.hasFieldOrPropertyWithValue("href",
							USER_MANAGER_URL + UserSettingsResource.SETTINGS_LINK_PATTERN.replace("{id}", UserTestFactory.ID_STR));
		}
	}

	@DisplayName("To Userprofile Resource List")
	@Nested
	class TestToUserProfileResourceList {
		private static final List<User> USERS = List.of(UserTestFactory.create(), UserTestFactory.create());

		@Test
		void shouldAddSelfLink() {
			var res = assembler.toUserProfileResourceList(USERS, USER_MANAGER_URL);

			assertThat(res.getLinks().get(UserProfileResourceAssembler.REL_SELF))
					.isNotNull()
					.hasFieldOrPropertyWithValue("href",
							USER_MANAGER_URL + UserProfileResource.USERS_PATH);
		}
	}
}